
/******************************************************************************
* MODULE     : rootless_disk.cpp
* DESCRIPTION: Find maximal radius of a rootless disk of an analytic function
* COPYRIGHT  : (C) 2005  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "analytic.hpp"
namespace mmx {
#define Positive exact_analytic::positive_type
#define Complex complex_ball
#define Series exact_analytic::series_type

template<> Positive
analytic_rep<exact_analytic>::root_free_disk (nat prec) const {
  Positive R = radius (prec);
  if (is_infinity (R)) R= Positive (10);
  Series s= germ ();
  nat i, n= 4 << prec;
  Complex* a= new Complex[n];
  for (i=0; i<n; i++)
    a[i]= s[i]->compute (prec * DEFAULT_LOW);
  Positive r  = (Positive (n) / Positive (n+1)) * R;
  Positive err= bound (r);
  Positive bnd= lower_bound_roots (a, n, r, err);
  delete[] a;
  return lower (bnd);
}

template<> Positive
analytic_rep<exact_analytic>::lower_bound (const Positive& r, nat prec) const {
  Positive R= radius (prec);
  if (is_infinity (R)) R= Positive (10);
  Series s= germ ();
  nat i, n= 4 << prec;
  Complex* a= new Complex[n];
  for (i=0; i<n; i++)
    a[i]= s[i]->compute (prec * DEFAULT_LOW);
  Positive bnd= minimum_disk (a, n, r);
  Positive mid= (Positive (n) / Positive (n+1)) * R;
  Positive err= bound (mid) * pow (r / mid, n);
  while (true) {
    if (Positive (4.0) * err < bnd) break;
    mid= Positive (0.8) * mid + Positive (0.2) * r;
    if (mid < Positive (1.2) * r) {
      delete[] a;
      return lower_bound (r, prec+1);
    }
    err= bound (mid) * pow (r / mid, n);
  }
  delete[] a;
  return lower (bnd - err);
}

#undef Positive
#undef Complex
#undef Series
} // namespace mmx
