
/******************************************************************************
* MODULE     : basix.cpp
* DESCRIPTION: Basic inclusions, definitions and routines
* COPYRIGHT  : (C) 2003  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "list.hpp"
#include "basix_utils.hpp"
namespace mmx {

/******************************************************************************
* Memory management
******************************************************************************/

static size_t mmx_bytes_in_use= 0;

void*
mmx_malloc (size_t new_size) {
  mmx_bytes_in_use += new_size;
  return malloc (new_size);
}

void*
mmx_realloc (void* ptr, size_t old_size, size_t new_size) {
  mmx_bytes_in_use += new_size - old_size;
  return realloc (ptr, new_size);
}

void
mmx_free (void* ptr, size_t old_size) {
  mmx_bytes_in_use -= old_size;
  free (ptr);
}

size_t
mmx_used_bytes () {
  return mmx_bytes_in_use;
}

/******************************************************************************
* Initialization
******************************************************************************/

static list<void_function> glob_inits;

global_initializer::global_initializer (void_function initializer) {
  glob_inits= cons (initializer, glob_inits);
}

void
mmx_init () {
  list<void_function> l= reverse (glob_inits);
  while (!nil (l)) {
    car (l) ();
    l= cdr (l);
  }
}

INITIALIZER(init_system);

/******************************************************************************
* Error management
******************************************************************************/

volatile void
fatal_error (char* message, char* routine, char* file) {
  cout << "\nFatal error: " << message
       << "\n             in '" << routine
       << "' in file '" << file << "'\n";
  exit (1);
}

} // namespace mmx
