
/******************************************************************************
* MODULE     : string.hpp
* DESCRIPTION: Strings with possible zero-characters
* COPYRIGHT  : (C) 2004  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __STRING_HPP
#define __STRING_HPP
#include "basix.hpp"
//#ifdef MMX_OLD_CPP
//#include <strstream>
//namespace std { typedef ostrstream ostringstream; }
//#else
//#include <sstream>
//#endif
namespace mmx {

class string;
class string_rep REP_STRUCT {
private:
  char* a;  // the string
  nat   n;  // length of string
  nat   l;  // allocated number of bytes

public:
  inline string_rep (nat n2): a ((char*) mmx_malloc (n2)), n(n2), l(n2) {}
  inline ~string_rep () { mmx_free ((void*) a, l); }
  void resize (nat n);
  void extend (nat n);

  friend class string;
  friend inline nat N (const string& s);
  friend inline char* S (string& s);
  friend inline const char* S (const string& s);
  friend string copy (const string& s);
  friend string operator * (const string& s, const string& t);
  friend int as_int (const string& s);
  friend double as_double (const string& s);
};

class string {
INDIRECT_PROTO (string, string_rep)
public:
  inline string (nat n=0): rep (new string_rep (n)) {}
  string (char c);
  string (const char *s);
  string (const char *s, nat n);

  inline friend nat N (const string& s) { return s.rep->n; }
  inline friend char* S (string& s) { s.secure (); return s.rep->a; }
  inline friend const char* S (const string& s) { return s.rep->a; }
  inline char operator [] (nat i) const { return rep->a[i]; }
  inline char& operator () (nat i) { secure (); return rep->a[i]; }
  string operator () (nat start, nat end) const;
  string& operator << (char c);
  string& operator << (const string& t);
  bool operator == (char* s) const;
  bool operator != (char* s) const;
  bool operator == (const string& s) const;
  bool operator != (const string& s) const;
  friend nat hash (const string& s);
  friend string copy (const string& s);
  friend ostream& operator << (ostream& out, const string& s);
  friend string operator * (const string& s, const string& t);
  friend string as_string (void* ptr);
  friend string as_string (int i);
  friend string as_string (unsigned int i);
  friend string as_string (long int i);
  friend string as_string (long unsigned int i);
  friend string as_string (float x);
  friend string as_string (double x);
  friend int as_int (const string& s);
  friend double as_double (const string& s);
  friend string unescape (const string& s);

  friend class string_rep;
};
INDIRECT_IMPL (string, string_rep)

string as_string (void* ptr);
string as_string (int i);
string as_string (unsigned int i);
string as_string (long int i);
string as_string (long unsigned int i);
string as_string (float x);
string as_string (double x);
int as_int (const string& s);
double as_double (const string& s);

inline bool eq (const string& s1, const string& s2) { return s1 == s2; }
inline bool neq (const string& s1, const string& s2) { return s1 != s2; }
bool operator <  (const string& s, const string& t);
bool operator <= (const string& s, const string& t);
bool operator >  (const string& s, const string& t);
bool operator >= (const string& s, const string& t);

char* as_charp (const string& s);
void free_charp (char* s);
string escape (const string& s);
string unescape (const string& s);
string quote (const string& s);
string unquote (const string& s);
string locase (const string& s);
string upcase (const string& s);
bool starts (const string& s, const string& what);
bool ends (const string& s, const string& what);
string make_range (const string& l, const string& r);
void decompose_range (const string& s, string& l, string& r);
string trunc_digits (const string& s, nat dd);

/******************************************************************************
* Print arbitrary objects to strings
******************************************************************************/

/*
template<class C> string
print_to_string (const C& data) {
  std::ostringstream out;
  out << data;
#ifdef MMX_OLD_CPP
  if (out.pcount () == 0) return "";
  else return string (out.str(), out.pcount ());
#else
  string s= out.str();
  return string ((char*) ((void*) s.data()), s.length());
#endif
}

template<class C> string
print_to_lisp (const C& data) {
  std::ostringstream out;
  print_lisp (out, data);
#ifdef MMX_OLD_CPP
  if (out.pcount () == 0) return "";
  else return string (out.str(), out.pcount ());
#else
  string s= out.str();
  return string ((char*) ((void*) s.data()), s.length());
#endif
}
*/

} // namespace mmx
#endif // __STRING_HPP
