
/******************************************************************************
* MODULE     : tuple.hpp
* DESCRIPTION: Syntactic tuple type constructor
* COPYRIGHT  : (C) 2006  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __TUPLE_HPP
#define __TUPLE_HPP
#include "array.hpp"
#include "table.hpp"
namespace mmx {
#define TMPL template<typename C>
#define Tuple tuple<C>

/******************************************************************************
* The tuple class
******************************************************************************/

template<typename C>
class tuple {
  generic rep;
public:
  inline tuple (const generic& g): rep (g) {}
  inline tuple (const Tuple& t): rep (t.rep) {}
  inline friend bool hard_eq (const Tuple& t, const Tuple& u) {
    return hard_eq (t.rep, u.rep); }
  inline friend bool hard_neq (const Tuple& t, const Tuple& u) {
    return hard_neq (t.rep, u.rep); }
  inline friend bool eq (const Tuple& t, const Tuple& u) {
    return eq (t.rep, u.rep); }
  inline friend bool neq (const Tuple& t, const Tuple& u) {
    return neq (t.rep, u.rep); }
  inline bool operator == (const Tuple& t) const { return rep == t.rep; }
  inline bool operator != (const Tuple& t) const { return rep != t.rep; }
  inline generic operator * () const { return rep; }
  inline C operator[] (nat i) const { return as<C> (rep[i]); }
};

TMPL inline nat hash (const Tuple& t) { return hash (*t); }
TMPL inline nat N (const Tuple& t) { return N (*t); }

TMPL generic
flatten (const Tuple& t) {
  return flatten (gen (MMX_TUPLE, cdr (compound_to_array (*t))));
}

PRINT_SUGAR(TMPL,Tuple)

/******************************************************************************
* Type information for tuple types
******************************************************************************/

TMPL struct type_information<Tuple > { static nat id; };
TMPL nat type_information<Tuple >::id=
  new_tuple_type_id (type_information<C>::id);

void tuple_type_info (nat& id, bool& mode);

inline bool
is_tuple_type (nat id) {
  bool mode= false;
  tuple_type_info (id, mode);
  return mode;
}

inline nat
tuple_to_scalar (nat id) {
  bool mode= false;
  tuple_type_info (id, mode);
  return id;
}

inline nat
new_tuple_type_id (nat id) {
  bool mode= true;
  tuple_type_info (id, mode);
  return id;
}

#undef TMPL
#undef Tuple
} // namespace mmx
#endif // __TUPLE_HPP
