
/******************************************************************************
* MODULE     : literal.hpp
* DESCRIPTION: Atomic literals
* COPYRIGHT  : (C) 2005  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __LITERAL_HPP
#define __LITERAL_HPP
#include "string.hpp"
namespace mmx {

class literal {
  string s;
public:
  inline literal (char *s2): s (s2) {}
  inline literal (const string& s2): s (s2) {}
  inline literal (const literal& s2): s (s2.s) {}
  inline friend bool hard_eq (const literal& l1, const literal& l2) {
    return hard_eq (l1.s, l2.s); }
  inline friend bool hard_neq (const literal& l1, const literal& l2) {
    return hard_neq (l1.s, l2.s); }
  inline friend bool eq (const literal& l1, const literal& l2) {
    return eq (l1.s, l2.s); }
  inline friend bool neq (const literal& l1, const literal& l2) {
    return neq (l1.s, l2.s); }
  inline bool operator == (const literal& s2) const { return s == s2.s; }
  inline bool operator != (const literal& s2) const { return s != s2.s; }
  inline string operator * () const { return s; }
  inline friend nat hash (const literal& s) { return hash (*s); }
};

generic flatten (const literal& s);
string literal_to_string (const generic& g);

template<typename C> C
make_literal (const literal& lit) {
  return C (*lit);
}

} // namespace mmx
#endif // __LITERAL_HPP
