
/******************************************************************************
* MODULE     : big_sum.hpp
* DESCRIPTION: Big commutative sums
* COPYRIGHT  : (C) 2006  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __BIG_SUM_HPP
#define __BIG_SUM_HPP
#include "mixed_symbol.hpp"
namespace mmx {
#define Table table<generic,generic>

/******************************************************************************
* Big sums (in reality linear combinations)
******************************************************************************/

class big_sum {
  Table sum;
public:
  inline big_sum (const Table& t): sum (t) {}
  inline big_sum (const big_sum& s): sum (s.sum) {}
  inline friend bool eq (const big_sum& p1, const big_sum& p2) {
    return eq (p1.sum, p2.sum); }
  inline friend bool neq (const big_sum& p1, const big_sum& p2) {
    return neq (p1.sum, p2.sum); }
  inline bool operator == (const big_sum& s) const { return sum == s.sum; }
  inline bool operator != (const big_sum& s) const { return sum != s.sum; }
  inline Table operator * () const { return sum; }
  inline friend nat hash (const big_sum& s) { return hash (*s); }
};

generic flatten (const big_sum& s);
Table sum_to_table (const generic& x);
generic table_to_sum (const Table& t);

inline big_sum melt (const big_sum& s) { return big_sum (melt (*s)); }
inline big_sum fuse (const big_sum& s) { return big_sum (fuse (*s)); }

/******************************************************************************
* Big sums as generics
******************************************************************************/

STMPL
struct internal<big_sum> {
  typedef mixed_symbol<big_sum> Repr;
  static inline Repr encode (const big_sum& x) { return Repr (x); }
  static inline big_sum decode (const Repr& x) { return *x; }
};

#undef Table
} // namespace mmx
#endif // __BIG_SUM_HPP
