
/******************************************************************************
* MODULE     : symbolix_glue.hpp
* DESCRIPTION: Standard glue for Mathemagix
* COPYRIGHT  : (C) 2006  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __SYMBOLIX_GLUE_HPP
#define __SYMBOLIX_GLUE_HPP
#include "category_glue.hpp"
#include "base_evaluator.hpp"
#include "literal.hpp"
#include "rational.hpp"
namespace mmx {

void define_commutative ();
void define_elementary ();
void define_calculus ();
void define_solve ();

template<typename C, typename T> generic
scalar_constructor (const T& x) {
  return as<generic> (as<C> (x));
}

template<typename Cond, typename C> void
define_symbolix () {
  define_constant_function<always,bool,generic,false> (generic ("scalar?"));
  define_constant_function<always,bool,integer,true> (generic ("scalar?"));
  define_constant_function<always,bool,rational,true> (generic ("scalar?"));
  define_constant_function<always,bool,C,true> (generic ("scalar?"));
  define_commutative ();
  define_elementary ();
  define_calculus ();
  define_solve ();
}

template<typename C> evaluator
symbolix_evaluator (const generic& name= "Scalar") {
  evaluator ev= base_evaluator ();
  select_evaluator (ev);
  define_prerequisites ();
  define_type<always,C> (name);
  define<always> ("literal_integer", make_literal<C>);
  define_constructor<always,integer> (scalar_constructor<C,integer>);
  define_constructor<always,rational> (scalar_constructor<C,rational>);
  define_ring<always,C> ();
  define_symbolix<always,C> ();
  restore_evaluator ();
  return ev;
}

} // namespace mmx
#endif // __SYMBOLIX_GLUE_HPP
